# SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://nvidia.github.io/NVTX/LICENSE.txt for license information.

from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, int32_t, int64_t
from libc.stddef cimport wchar_t

cdef extern from "nvtx3/nvToolsExt.h" nogil:

    cdef int NVTX_VERSION
    cdef int NVTX_EVENT_ATTRIB_STRUCT_SIZE

    cdef void nvtxInitialize(const void* reserved)

    cdef struct nvtxDomainRegistration_st:
        pass
    ctypedef struct nvtxDomainRegistration_st:
        pass
    ctypedef nvtxDomainRegistration_st nvtxDomainRegistration
    ctypedef nvtxDomainRegistration* nvtxDomainHandle_t

    cdef struct nvtxStringRegistration_st:
        pass
    ctypedef nvtxStringRegistration_st nvtxStringRegistration
    ctypedef nvtxStringRegistration* nvtxStringHandle_t

    ctypedef uint64_t nvtxRangeId_t

    ctypedef enum nvtxColorType_t:
        NVTX_COLOR_UNKNOWN = 0
        NVTX_COLOR_ARGB = 1

    ctypedef enum nvtxMessageType_t:
        NVTX_MESSAGE_UNKNOWN = 0
        NVTX_MESSAGE_TYPE_ASCII = 1
        NVTX_MESSAGE_TYPE_UNICODE = 2
        NVTX_MESSAGE_TYPE_REGISTERED = 3

    ctypedef union nvtxMessageValue_t:
        const char* ascii
        const wchar_t* unicode
        nvtxStringHandle_t registered

    ctypedef enum nvtxPayloadType_t:
        NVTX_PAYLOAD_UNKNOWN = 0
        NVTX_PAYLOAD_TYPE_UNSIGNED_INT64 = 1
        NVTX_PAYLOAD_TYPE_INT64 = 2
        NVTX_PAYLOAD_TYPE_DOUBLE = 3
        NVTX_PAYLOAD_TYPE_UNSIGNED_INT32 = 4
        NVTX_PAYLOAD_TYPE_INT32 = 5
        NVTX_PAYLOAD_TYPE_FLOAT = 6

    cdef union payload_t:
        uint64_t ullValue
        int64_t llValue
        double dValue
        uint32_t uiValue
        int32_t iValue
        float fValue

    ctypedef struct nvtxEventAttributes_v2:
        uint16_t version
        uint16_t size
        uint32_t category
        int32_t colorType
        uint32_t color
        int32_t payloadType
        payload_t payload
        int32_t messageType
        nvtxMessageValue_t message

    ctypedef nvtxEventAttributes_v2 nvtxEventAttributes_t

    cdef nvtxDomainHandle_t nvtxDomainCreateA(const char* name)
    cdef void nvtxDomainDestroy(nvtxDomainHandle_t domain)

    cdef nvtxStringHandle_t nvtxDomainRegisterStringA(nvtxDomainHandle_t domain, const char* string)

    cdef int nvtxDomainRangePushEx(
        nvtxDomainHandle_t domain,
        const nvtxEventAttributes_t* eventAttrib
    )
    cdef int nvtxDomainRangePop(nvtxDomainHandle_t domain)

    cdef nvtxRangeId_t nvtxDomainRangeStartEx(
        nvtxDomainHandle_t domain,
        const nvtxEventAttributes_t* eventAttrib
    )
    cdef void nvtxDomainRangeEnd(
        nvtxDomainHandle_t domain,
        nvtxRangeId_t
    )

    cdef void nvtxDomainMarkEx(
        nvtxDomainHandle_t domain,
        const nvtxEventAttributes_t* eventAttrib
    )

    cdef void nvtxDomainNameCategoryA(
        nvtxDomainHandle_t domain,
        uint32_t category,
        const char* name
    )

cdef extern from "nvtx3/nvToolsExtPayload.h" nogil:
    cdef uint8_t nvtxDomainIsEnabled(nvtxDomainHandle_t domain)


cdef class EventAttributes:
    cdef object _message
    cdef object _color
    cdef uint32_t _category
    cdef object _payload
    cdef nvtxStringHandle_t string_handle
    cdef nvtxEventAttributes_t c_obj


cdef class DomainHandle:
    cdef bytes _name
    cdef nvtxDomainHandle_t c_obj

cdef class StringHandle:
    cdef bytes _string
    cdef nvtxStringHandle_t c_obj



# SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://nvidia.github.io/NVTX/LICENSE.txt for license information.

from functools import lru_cache

from nvtx._lib.lib cimport *
from nvtx.colors import color_to_hex

from typing import Optional


cpdef bytes _to_bytes(object s):
    return s if isinstance(s, bytes) else s.encode()

def initialize():
    nvtxInitialize(NULL)


cdef class EventAttributes:
    """
    A wrapper class for ``nvtxEventAttributes_t`` C struct.
    Use :func:`nvtx.Domain.get_event_attributes` to create an instance.

    Attributes
    ----------
    message : RegisteredString
        A message associated with the event.
        Retrieved by :func:`nvtx.Domain.get_registered_string`.
    color : str or int
        A color associated with the event.
        Supports `matplotlib` colors if it is available.
    category : int
        An integer specifying the category within the domain
        under which the event is scoped.
        If not set, the event is not associated with a category.
        Retrieved by :func:`nvtx.Domain.get_category_id`.
    payload : int
        A numeric value to be associated with this event.
    """

    def __init__(self, object message=None, color=None, category=None, payload=None):
        self.c_obj = nvtxEventAttributes_t(0)
        self.c_obj.version = NVTX_VERSION
        self.c_obj.size = NVTX_EVENT_ATTRIB_STRUCT_SIZE
        self.c_obj.colorType = NVTX_COLOR_ARGB
        self.c_obj.messageType = NVTX_MESSAGE_TYPE_REGISTERED

        self.message = message
        self.color = color
        self.category = category
        self.payload = payload

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, object value):
        self._message = value
        self.c_obj.message.registered = (<StringHandle> self._message.handle).c_obj

    @property
    def color(self):
        return self._color

    @color.setter
    def color(self, value):
        self._color = value
        self.c_obj.color = color_to_hex(self._color)

    @property
    def category(self):
        return self._category

    @category.setter
    def category(self, value):
        if value is not None:
            self._category = value
            self.c_obj.category = value

    @property
    def payload(self):
        return self._payload

    @payload.setter
    def payload(self, value):
        if value is not None:
            self._payload = value

            if isinstance(self._payload, int):
                self.c_obj.payload.llValue = self._payload
                self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_INT64
            elif isinstance(self._payload, float):
                self.c_obj.payload.dValue = self._payload
                self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_DOUBLE
            else:
                raise RuntimeError('Payload must be int or float')


cdef class DomainHandle:

    def __init__(self, object name=None):
        if name is not None:
            self._name = _to_bytes(name)
            self.c_obj = nvtxDomainCreateA(
                self._name
            )
        else:
            self._name = b""
            self.c_obj = NULL

    @property
    def name(self):
        return self._name.decode()

    def enabled(self):
        return bool(nvtxDomainIsEnabled(self.c_obj))

    def __dealloc__(self):
        nvtxDomainDestroy(self.c_obj)


class RegisteredString:
    """
    A wrapper class for ``nvtxStringHandle_t`` C struct.
    Use :func:`nvtx.Domain.get_registered_string` to create an instance.
    """
    def __init__(self, domain, string=None):
        self.string = string
        self.domain = domain
        self.handle = StringHandle(domain, string)

class DummyDomain:
    """
    A replacement for :class:`nvtx.Domain` when the domain is disabled.
    (e.g., when no tool is attached).
    """
    def get_registered_string(self, string):
        pass

    def get_category_id(self, name):
        pass

    def get_event_attributes(self, message=None, color=None, category=None, payload=None):
        pass

    def mark(self, EventAttributes attributes):
        pass

    def push_range(self, EventAttributes attributes):
        pass

    def pop_range(self):
        pass

    def start_range(self, EventAttributes attributes):
        return 0

    def end_range(self, nvtxRangeId_t range_id):
        pass


dummy_domain = DummyDomain()

class Domain:
    """
    A class that provides an interface to NVTX API per domain,
    and produces less overhead than using the global functions from ``nvtx`` module.

    Notes
    -----
    - Use :func:`nvtx.get_domain` to create an instance.
    - If the domain is disabled (e.g., when no tool is attached),
      the instance returned is a :class:`nvtx._lib.lib.DummyDomain`.
    """

    def __new__(cls, name: Optional[str] = None):
        handle = DomainHandle(name)
        if handle.enabled():
            obj = super().__new__(cls)
            obj.handle = handle
            return obj
        else:
            return dummy_domain

    def __init__(self, name: Optional[str] = None):
        self.name = name
        self.categories = {}

    @lru_cache(maxsize=None)
    def get_registered_string(self, string) -> RegisteredString:
        """
        Register a given string under this domain (on first use), and return the handle.

        Parameters
        ----------
        string : str
            The string to be registered.
        """
        return RegisteredString(self.handle, string)

    @lru_cache(maxsize=None)
    def get_category_id(self, name) -> int:
        """
        Returns the category ID corresponding to the category `name`.
        On first use with a specific `name`, a new ID is assigned with the given name.

        Parameters
        ----------
        name : str
            The name of the category.
        """
        cdef DomainHandle dh = self.handle
        category_id = len(self.categories) + 1
        self.categories[name] = category_id
        nvtxDomainNameCategoryA(
            dh.c_obj,
            category_id,
            _to_bytes(name)
        )
        return category_id

    @lru_cache(maxsize=None)
    def get_event_attributes(self, message=None, color=None, category=None, payload=None
            )-> EventAttributes:
        """
        Get or create an :class:`nvtx._lib.lib.EventAttributes` object.
        The results of this function are cached, i.e.,
        the same object is returned for the same parameters.

        Parameters
        ----------
        message : str
            A message associated with the event.
        color : str, int, optional
            A color associated with the event.
            Supports `matplotlib` colors if it is available.
        category : str, int, optional
            A string or an integer specifying the category within the domain under which the event
            is scoped. If unspecified, the event is not associated with a category.
        payload : int, float, optional
            A numeric value to be associated with this event.
        """
        if isinstance(category, str):
            category = self.get_category_id(category)
        return EventAttributes(self.get_registered_string(message), color, category, payload)

    def mark(self, EventAttributes attributes):
        """
        Mark an instantaneous event.

        Parameters
        ----------
        attributes : EventAttributes
            The event attributes to be associated with the event.

        Examples
        --------
        >>> import nvtx
        >>> domain = nvtx.Domain('my_domain')
        >>> attributes = domain.get_event_attributes(message='my_marker')
        >>> domain.mark(attributes)
        """
        nvtxDomainMarkEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)

    def push_range(self, EventAttributes attributes):
        """
        Mark the beginning of a code range.

        Parameters
        ----------
        attributes: EventAttributes
            The event attributes to be associated with the range.
        
        Notes
        -----
        When applicable, prefer to use :class:`annotate`.

        Examples
        --------
        >>> import time
        >>> import nvtx
        >>> domain = nvtx.Domain('my_domain')
        >>> attributes = domain.get_event_attributes(message='my_code_range')
        >>> domain.push_range(attributes)
        >>> time.sleep(1)
        >>> domain.pop_range()
        """
        nvtxDomainRangePushEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)

    def pop_range(self):
        """
        Mark the end of a code range that was started with :func:`Domain.push_range`.
        """
        nvtxDomainRangePop((<DomainHandle>self.handle).c_obj)

    def start_range(self, EventAttributes attributes) -> int:
        """
        Mark the beginning of a process range.

        Parameters
        ----------
        attributes : EventAttributes
            The event attributes to be associated with the range.

        Returns
        -------
        A numeric value that must be passed to :func:`Domain.end_range`.

        Examples
        --------
        >>> import time
        >>> import nvtx
        >>> domain = nvtx.Domain('my_domain')
        >>> attributes = domain.get_event_attributes(message='my_code_range')
        >>> range_id = domain.start_range(attributes)
        >>> time.sleep(1)
        >>> domain.end_range(range_id)
        """
        return nvtxDomainRangeStartEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)

    def end_range(self, nvtxRangeId_t range_id):
        """
        Mark the end of a process range that was started with :func:`Domain.start_range`.

        Parameters
        ----------
        range_id : int
            The value returned by :func:`Domain.start_range`.
        """
        nvtxDomainRangeEnd((<DomainHandle>self.handle).c_obj, range_id)
cdef class StringHandle:

    def __init__(self, DomainHandle domain_handle, object string=None):
        if string is not None:
            self._string = _to_bytes(string)
            self.c_obj = nvtxDomainRegisterStringA(
                domain_handle.c_obj, self._string
            )
        else:
            self._string = b""
            self.c_obj = NULL

    @property
    def string(self):
        return self._string.decode()


def push_range(EventAttributes attributes, DomainHandle domain):
    nvtxDomainRangePushEx(domain.c_obj, &attributes.c_obj)


def pop_range(DomainHandle domain):
    nvtxDomainRangePop(domain.c_obj)


def start_range(EventAttributes attributes, DomainHandle domain):
    return nvtxDomainRangeStartEx(domain.c_obj, &attributes.c_obj), domain


def end_range(nvtxRangeId_t range_id, DomainHandle domain):
    nvtxDomainRangeEnd(domain.c_obj, range_id)


def mark(EventAttributes attributes, DomainHandle domain):
    nvtxDomainMarkEx(domain.c_obj, &attributes.c_obj)



# SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://nvidia.github.io/NVTX/LICENSE.txt for license information.

import functools

_NVTX_COLORS = {
    None: 0x0000FF,  # blue
    "green": 0x008000,
    "blue": 0x0000FF,
    "yellow": 0xFFFF00,
    "purple": 0x800080,
    "rapids": 0x7400FF,
    "cyan": 0x00FFFF,
    "red": 0xFF0000,
    "white": 0xFFFFFF,
    "darkgreen": 0x006400,
    "orange": 0xFFA500,
}


@functools.lru_cache()
def color_to_hex(color=None):
    """
    Convert color to ARGB hex value.
    """
    if isinstance(color, int):
        return color
    if color in _NVTX_COLORS:
        return _NVTX_COLORS[color]
    try:
        import matplotlib.colors
    except ImportError as e:
        raise TypeError(
            f"Invalid color {color}. Please install matplotlib "
            "for additional colors support"
        ) from e
    rgba = matplotlib.colors.to_rgba(color)
    argb = (rgba[-1], rgba[0], rgba[1], rgba[2])
    return int(matplotlib.colors.to_hex(argb, keep_alpha=True)[1:], 16)



# SPDX-FileCopyrightText: Copyright (c) 2021-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://nvidia.github.io/NVTX/LICENSE.txt for license information.

from nvtx._lib.lib import _to_bytes
from nvtx._lib.lib cimport *
from nvtx._lib.profiler cimport Profile


def test_profiler_message():
    # test that when we call a function foo(), the name of the
    # function is captured correctly.

    prof = Profile(linenos=False)

    def foo():
        assert prof.__attrib.message.registered == nvtxDomainRegisterStringA(
            prof.__domain, _to_bytes("foo"))

    prof.enable()
    foo()
    prof.disable()



# SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://nvidia.github.io/NVTX/LICENSE.txt for license information.

from nvtx._lib.lib import (
    Domain,
    DummyDomain,
    dummy_domain,
    RegisteredString,
    EventAttributes,
    pop_range,
    push_range,
    start_range,
    end_range,
    mark,
)



import sys
from runpy import run_path
from optparse import OptionParser


def main():
    from nvtx import Profile

    usage = "%prog [options] scriptfile [args] ..."

    parser = OptionParser(usage)
    parser.add_option(
        "--linenos",
        action="store_true",
        dest="linenos",
        default=True,
        help="Include file and line number information in annotations. Otherwise, "
        "only the function name is used.",
    )
    parser.add_option(
        "--no-linenos",
        action="store_false",
        dest="linenos",
        default=True,
        help="Do not include file and line number information in annotations.",
    )
    parser.add_option(
        "--annotate-cfuncs",
        action="store_true",
        dest="annotate_cfuncs",
        default=False,
        help="Also annotate C-extension and builtin functions. [default: %default]",
    )

    options, args = parser.parse_args()
    script_file = args[0]

    sys.argv = args
    profiler = Profile(linenos=options.linenos, annotate_cfuncs=options.annotate_cfuncs)

    profiler.enable()

    try:
        run_path(script_file, run_name='__main__')
    finally:
        profiler.disable()


if __name__ == "__main__":
    main()



import os
import sys
import threading

from nvtx._lib.lib import _to_bytes
from nvtx.colors import _NVTX_COLORS

from nvtx._lib.lib cimport *

DEFAULT_COLOR = _NVTX_COLORS[None]

cdef class Profile:
    """
    Class for programmatically controlling NVTX automatic annotations.

    Parameters
    ----------
    linenos
        Include file and line number information in annotations.
    annotate_cfuncs
        Also annotate C-extensions and builtin functions.

    Examples
    --------
    >>> import nvtx
    >>> import time
    >>> pr = nvtx.Profile()
    >>> pr.enable()
    >>> time.sleep(1) # this call to `sleep` is captured by nvtx.
    >>> pr.disable()
    >>> time.sleep(1) # this one is not.
    """
    def __init__(self, linenos: bool = True, annotate_cfuncs: bool = True):
        
        self.linenos = linenos
        self.annotate_cfuncs = annotate_cfuncs
        self.__domain = nvtxDomainCreateA(b"nvtx.py")
        self.__attrib = nvtxEventAttributes_t(0)
        self.__attrib.version = NVTX_VERSION
        self.__attrib.size = NVTX_EVENT_ATTRIB_STRUCT_SIZE
        self.__attrib.colorType = NVTX_COLOR_ARGB
        self.__attrib.color = DEFAULT_COLOR
        self.__attrib.messageType = NVTX_MESSAGE_TYPE_REGISTERED

    def _profile(self, frame, event, arg):
        # profile function meant to be used with sys.setprofile
        if event == "call":
            name = frame.f_code.co_name
            if self.linenos:
                fname = os.path.basename(frame.f_code.co_filename)
                lineno = frame.f_lineno
                message = f"{fname}:{lineno}({name})"
            else:
                message = name
            self.push_range(message)
        elif event == "c_call" and self.annotate_cfuncs:
            self.push_range(arg.__name__)
        elif event == "return":
            self.pop_range()
        elif event in {"c_return", "c_exception"} and self.annotate_cfuncs:
            self.pop_range()
        return None

    cdef push_range(self, message):
        self.__attrib.message.registered = nvtxDomainRegisterStringA(
            self.__domain, _to_bytes(message))
        nvtxDomainRangePushEx(self.__domain, &self.__attrib)

    cdef pop_range(self):
        nvtxDomainRangePop(self.__domain)

    def enable(self):
        """Start annotating function calls automatically.
        """
        threading.setprofile(self._profile)
        sys.setprofile(self._profile)

    def disable(self):
        """Stop annotating function calls automatically.
        """
        threading.setprofile(None)
        sys.setprofile(None)



# SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://nvidia.github.io/NVTX/LICENSE.txt for license information.

import contextlib
import os

from functools import wraps, lru_cache
from typing import Optional, Union, Tuple

from nvtx._lib import (
    Domain,
    DummyDomain,
    dummy_domain,
    mark as libnvtx_mark,
    pop_range as libnvtx_pop_range,
    push_range as libnvtx_push_range,
    start_range as libnvtx_start_range,
    end_range as libnvtx_end_range,
)

_ENABLED = not os.getenv("NVTX_DISABLE", False)


@lru_cache(maxsize=None)
def get_domain(name: Optional[str] = None) -> Union[Domain, DummyDomain]:
    """
    Get or create a :class:`Domain` object for a domain name.
    """
    return Domain(name)


class annotate:
    """
    Annotate code ranges using a context manager or a decorator.

    Parameters
    ----------
    message
        A message associated with the annotated code range.
        When used as a decorator, defaults to the decorated function name.
        When used as a context manager, defaults to the empty string.
        Messages are cached and are registered as Registered Strings
        in NVTX.
        Caching a very large number of messages may lead to increased
        memory usage.
    color
        A color associated with the annotated code range.
        Supports `matplotlib` colors if it is available.
    domain
        A string specifying the domain under which the code range is
        scoped. The default domain is named "NVTX".
    category
        A string or an integer specifying the category within the domain
        under which the code range is scoped. If unspecified, the code
        range is not associated with a category.
    payload
        A numeric value to be associated with this event.

    Examples
    --------
    >>> import nvtx
    >>> import time

    Using a decorator:

    >>> @nvtx.annotate(color="red", domain="my_domain"):    # `message` defaults to "func"
    ... def func():
    ...     time.sleep(0.1)

    Using a context manager:

    >>> with nvtx.annotate("my_code_range", color="blue"):
    ...    time.sleep(10)
    """

    def __init__(self, message: Optional[str] = None, color: Optional[Union[str, int]] = None,
                 domain: Optional[str] = None, category: Optional[Union[str, int]] = None,
                 payload: Optional[Union[int, float]] = None):

        self.init_args = message, color, domain, category, payload
        self.domain = get_domain(domain)
        if self.domain is not dummy_domain:
            self.attributes = self.domain.get_event_attributes(message, color, category, payload)

    def __reduce__(self):
        return self.__class__, self.init_args

    def __enter__(self):
        if self.domain is not dummy_domain:
            libnvtx_push_range(self.attributes, self.domain.handle)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self.domain is not dummy_domain:
            libnvtx_pop_range(self.domain.handle)
        return False

    def __call__(self, func):
        if self.domain is dummy_domain:
            return func
        if not self.attributes.message.string:
            self.attributes.message = self.domain.get_registered_string(func.__name__)

        @wraps(func)
        def inner(*args, **kwargs):
            libnvtx_push_range(self.attributes, self.domain.handle)
            result = func(*args, **kwargs)
            libnvtx_pop_range(self.domain.handle)
            return result

        return inner


def mark(message: Optional[str] = None, color: Optional[Union[str, int]] = "blue",
         domain: Optional[str] = None, category: Optional[Union[str, int]] = None,
         payload: Optional[Union[int, float]] = None):
    """
    Mark an instantaneous event.

    Parameters
    ----------
    message
        A message associated with the event.
        Messages are cached and are registered as Registered Strings
        in NVTX.
        Caching a very large number of messages may lead to increased
        memory usage.
    color
        Color associated with the event.
        Supports `matplotlib` colors if it is available.
    domain
        A string specifying the domain under which the event is scoped.
        The default domain is named "NVTX".
    category
        A string or an integer specifying the category within the domain
        under which the event is scoped. If unspecified, the event is
        not associated with a category.
    payload
        A numeric value to be associated with this event

    Notes
    -----
    For best performance, use :func:`Domain.mark`.

    Examples
    --------
    >>> import nvtx
    >>> nvtx.mark("my_marker", domain="my_domain")
    """
    domain = get_domain(domain)
    if domain is not dummy_domain:
        libnvtx_mark(domain.get_event_attributes(message, color, category, payload), domain.handle)


def push_range(message: Optional[str] = None, color: Optional[Union[str, int]] = "blue",
               domain: Optional[str] = None, category: Optional[Union[str, int]] = None,
               payload: Optional[Union[int, float]] = None):
    """
    Mark the beginning of a code range.

    Parameters
    ----------
    message
        A message associated with the annotated code range.
        Messages are cached and are registered as Registered Strings
        in NVTX.
        Caching a very large number of messages may lead to increased
        memory usage.
    color
        A color associated with the annotated code range.
        Supports `matplotlib` colors if it is available.
    domain
        Name of a domain under which the code range is scoped.
        The default domain name is "NVTX".
    category
        A string or an integer specifying the category within the domain
        under which the code range is scoped. If unspecified, the code range
        is not associated with a category.
    payload
        A numeric value to be associated with this event

    Notes
    -----
    When applicable, prefer to use :class:`annotate`.
    Otherwise, for best performance, use :func:`Domain.push_range` and :func:`Domain.pop_range`.

    Examples
    --------
    >>> import time
    >>> import nvtx
    >>> nvtx.push_range("my_code_range", domain="my_domain")
    >>> time.sleep(1)
    >>> nvtx.pop_range(domain="my_domain")
    """
    domain = get_domain(domain)
    if domain is not dummy_domain:
        libnvtx_push_range(domain.get_event_attributes(message, color, category, payload),
                           domain.handle)


def pop_range(domain: Optional[str] = None):
    """
    Mark the end of a code range that was started with :func:`push_range`.

    Parameters
    ----------
    domain
        The domain under which the code range is scoped. The default
        domain is "NVTX".
    """
    domain = get_domain(domain)
    if domain is not dummy_domain:
        libnvtx_pop_range(domain.handle)


def start_range(message: Optional[str] = None, color: Optional[Union[str, int]] = None,
                domain: Optional[str] = None, category: Optional[Union[str, int]] = None,
                payload: Optional[Union[int, float]] = None) -> Tuple[int, int]:
    """
    Mark the beginning of a process range.

    Parameters
    ----------
    message
        A message associated with the range.
        Messages are cached and are registered as Registered Strings
        in NVTX.
        Caching a very large number of messages may lead to increased
        memory usage.
    color
        A color associated with the range.
        Supports `matplotlib` colors if it is available.
    domain
        Name of a domain under which the range is scoped.
        The default domain name is "NVTX".
    category
        A string or an integer specifying the category within the domain
        under which the range is scoped. If unspecified, the range
        is not associated with a category.
    payload
            A numeric value to be associated with this event

    Returns
    -------
    A tuple of the form ``(range_id, domain_handle)`` that must be passed to :func:`end_range`.

    Notes
    -----
    For best performance, use :func:`Domain.start_range` and :func:`Domain.end_range`.

    Examples
    --------
    >>> import time
    >>> import nvtx
    >>> range_id = nvtx.start_range("my_code_range", domain="my_domain")
    >>> time.sleep(1)
    >>> nvtx.end_range(range_id)
    """
    domain = get_domain(domain)
    if domain is not dummy_domain:
        return libnvtx_start_range(
            domain.get_event_attributes(message, color, category, payload), domain.handle)


def end_range(range_id: Tuple[int, int]):
    """
    Mark the end of a code range that was started with :func:`start_range`.

    Parameters
    ----------
    range_id
        The tuple object returned by :func:`start_range`.
    """
    if range_id is not None:
        libnvtx_end_range(*range_id)


def enabled():
    """
    Returns True if nvtx is enabled.
    """
    return _ENABLED


if not enabled():

    class annotate(contextlib.nullcontext):
        def __init__(self, *args, **kwargs):
            super().__init__()

        def __call__(self, func):
            return func

    # Could use a decorator here but overheads are significant enough
    # not to. See https://github.com/NVIDIA/NVTX/pull/24 for discussion.
    def mark(message=None, color=None, domain=None, category=None, payload=None):
        pass

    def push_range(message=None, color=None, domain=None, category=None, payload=None):
        pass

    def pop_range(domain=None):
        pass

    def start_range(message=None, color=None, domain=None, category=None, payload=None):
        pass

    def end_range(range_id):
        pass



# SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://nvidia.github.io/NVTX/LICENSE.txt for license information.

import pickle

import pytest

import nvtx
from nvtx.nvtx import dummy_domain


@pytest.mark.parametrize(
    "message",
    [
        None,
        "",
        "x",
        "abc",
        "abc def"
    ]
)
@pytest.mark.parametrize(
    "color",
    [
        None,
        "red",
        "green",
        "blue"
    ]
)
@pytest.mark.parametrize(
    "domain",
    [
        None,
        "",
        "x",
        "abc",
        "abc def"
    ]
)
@pytest.mark.parametrize(
    "payload",
    [
        None,
        1,
        1.0
    ]
)
def test_annotate_context_manager(message, color, domain, payload):
    with nvtx.annotate(message=message, color=color, domain=domain, payload=payload):
        pass


@pytest.mark.parametrize(
    "message",
    [
        None,
        "",
        "x",
        "abc",
        "abc def"
    ]
)
@pytest.mark.parametrize(
    "color",
    [
        None,
        "red",
        "green",
        "blue"
    ]
)
@pytest.mark.parametrize(
    "domain",
    [
        None,
        "",
        "x",
        "abc",
        "abc def"
    ]
)
@pytest.mark.parametrize(
    "payload",
    [
        None,
        1,
        1.0
    ]
)
def test_annotate_decorator(message, color, domain, payload):
    @nvtx.annotate(message=message, color=color, domain=domain, payload=payload)
    def foo():
        pass

    foo()


def test_pickle_annotate():
    orig = nvtx.annotate(message="foo", color="blue", domain="test")
    pickled = pickle.dumps(orig)
    unpickled = pickle.loads(pickled)

    assert orig.init_args == unpickled.init_args


def test_disabled_domain():
    assert nvtx.get_domain("x") is dummy_domain


@pytest.mark.parametrize(
    "message",
    [
        None,
        "",
        "x",
        "abc",
        "abc def"
    ]
)
@pytest.mark.parametrize(
    "color",
    [
        None,
        "red",
        "green",
        "blue"
    ]
)
@pytest.mark.parametrize(
    "domain",
    [
        None,
        "",
        "x",
        "abc",
        "abc def"
    ]
)
@pytest.mark.parametrize(
    "category",
    [
        None,
        "",
        "y"
        "x",
        "abc",
        "abc def",
        0,
        1,
    ]
)
def test_categories_basic(message, color, domain, category):
    with nvtx.annotate(message=message, domain=domain, category=category):
        pass


@pytest.mark.parametrize(
    "message",
    [
        None,
        "abc",
    ]
)
@pytest.mark.parametrize(
    "color",
    [
        None,
        "red",
    ]
)
@pytest.mark.parametrize(
    "domain",
    [
        None,
        "abc",
    ]
)
@pytest.mark.parametrize(
    "category",
    [
        None,
        "abc",
        1,
    ]
)
@pytest.mark.parametrize(
    "payload",
    [
        None,
        1,
        1.0
    ]
)
def test_start_end(message, color, domain, category, payload):
    rng = nvtx.start_range(message, color, domain, category, payload)
    nvtx.end_range(rng)

    domain = nvtx.get_domain(domain)
    attributes = domain.get_event_attributes(message, color, category, payload)
    domain.end_range(domain.start_range(attributes))




@pytest.mark.parametrize(
    "message",
    [
        None,
        "abc",
    ]
)
@pytest.mark.parametrize(
    "color",
    [
        None,
        "red",
    ]
)
@pytest.mark.parametrize(
    "domain",
    [
        None,
        "abc",
    ]
)
@pytest.mark.parametrize(
    "category",
    [
        None,
        "abc",
        1,
    ]
)
@pytest.mark.parametrize(
    "payload",
    [
        None,
        1,
        1.0
    ]
)
def test_push_pop(message, color, domain, category, payload):
    nvtx.push_range(message, color, domain, category, payload)
    nvtx.pop_range()

    domain = nvtx.get_domain(domain)
    attributes = domain.get_event_attributes(message, color, category, payload)
    domain.push_range(attributes)
    domain.pop_range()


@pytest.mark.parametrize(
    "message",
    [
        None,
        "abc",
    ]
)
@pytest.mark.parametrize(
    "color",
    [
        None,
        "red",
    ]
)
@pytest.mark.parametrize(
    "domain",
    [
        None,
        "abc",
    ]
)
@pytest.mark.parametrize(
    "category",
    [
        None,
        "abc",
        1,
    ]
)
@pytest.mark.parametrize(
    "payload",
    [
        None,
        1,
        1.0
    ]
)
def test_mark(message, color, domain, category, payload):
    nvtx.mark(message, color, domain, category, payload)

    domain = nvtx.get_domain(domain)
    attributes = domain.get_event_attributes(message, color, category, payload)
    domain.mark(attributes)


def test_domain_disabled_no_func_annotation():
    def foo():
        pass

    assert nvtx.annotate()(foo) is foo



# SPDX-FileCopyrightText: Copyright (c) 2020-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://nvidia.github.io/NVTX/LICENSE.txt for license information.

import shutil

from Cython.Build import cythonize
from pathlib import Path
from setuptools import setup
from setuptools.command.sdist import sdist
from setuptools.extension import Extension


# ../c/include
c_include_path = Path(__file__).parent.parent / 'c' / 'include'

# When building from source distribution (.tar.gz), ./include dir exists (added by sdist command)
# Otherwise, we are building from sources, so we need to use `c_include_path`
include_dirs = ['include' if Path('include').exists() else str(c_include_path)]


class NvtxSdist(sdist):
    def run(self):
        try:
            shutil.copytree(c_include_path, 'include')
            super().run()
        finally:
            shutil.rmtree('include', ignore_errors=True)


setup(
    cmdclass=dict(sdist=NvtxSdist),
    ext_modules=cythonize(
        Extension('*', sources=['src/nvtx/_lib/*.pyx'], include_dirs=include_dirs),
        compiler_directives=dict(language_level=3, embedsignature=True))
)
